#!BPY
################
#author: Daniel Jungmann (dsj at gmx dot net)
#version: 2.5
################
"""
Name: 'Eternal-Lands files (.e3d)'
Blender: 243
Group: 'Import'
Tip: 'Import from Eternal-Lands 3d object files.'
"""
import Blender
from Blender import NMesh, Image, Object, Material, Texture, Mesh
from Blender.BGL import *
from Blender.Draw import *
from Blender.Window import *
from Blender.Image import *
from Blender.Mathutils import Vector
import sys, struct, string
from types import *
import os
from os import path
import md5
import gzip
from tempfile import gettempdir

path = Blender.Get('uscriptsdir')
if not path:
	path = Blender.Get('scriptsdir')
if not path:
	path = Blender.Get('filename')
sys.path.append(path)
import el3d
from el3d import *
from el3d.optimizer import *
from el3d.compress import *
from el3d.el3d_file_obj import *

default_uv_layer = "UVTex"
extra_uv_layer = "Extra UVTex"

def vertex_hash(v):
	return struct.pack("3f", v[0], v[1], v[2])

def normals_hash(n):
	return struct.pack("3i", n[0]*32768, n[1]*32768, n[2]*32768)

def load_el3d(filename):
	el3d_file_obj = el3d_file()
	print "Starting e3d import from file: ", filename
	compressed = True
	try:
		file = gzip.open(filename, 'rb')
		try:
			el3d_file_obj.load(file)
			compressed = True
		except IOError:
			compressed = False
			file.close()
	except IOError:
		compressed = False
	if (not compressed):
		try:
			file = open(filename, 'rb')
		except IOError, (errno, strerror):
			errmsg = "IOError #%s: %s" % (errno, strerror)
			Blender.Draw.PupMenu(errmsg)
			Exit()
			return
		try:
			el3d_file_obj.load(file)
		except IOError, (errno, strerror):
			errmsg = "IOError #%s: %s" % (errno, strerror)
			Blender.Draw.PupMenu(errmsg)
			Exit()
			return	
	fname = Blender.sys.basename(filename)
	name, ext = fname.rsplit('.', 1)
	if (ext == "gz"):
		fname = name
		name, ext = fname.rsplit('.', 1)
	mesh = Mesh.New(name)

	extend_list = []
	vert_dict = {}
	vert_no = 0
	#make the vertices
	for v in el3d_file_obj.vertex_list:
		hash_value = vertex_hash(v.position)
		if not vert_dict.has_key(hash_value):
			vert_dict[hash_value] = vert_no
			vert_no += 1
			extend_list.append(v.position)

	mesh.verts.extend([Vector()])
	mesh.verts.extend(extend_list)
	
	options = extract_options(el3d_file_obj.options)

	has_normal = options[0]
	has_tangent = options[1]
	has_extra_uv = options[2]
	has_color = options[3]

	#read materials
	faces = []
	for x in range(0, el3d_file_obj.header.material_no):
		material = el3d_file_obj.material_list[x]
		fname = Blender.sys.dirname(filename) + Blender.sys.dirsep + material.material_name
		try:
			img = Blender.Image.Load(fname)
			compress = False
		except IOError:
			compress = True
		if (compress):
			tmp_name = gettempdir() + Blender.sys.dirsep + material.material_name
			try:
				tmpfile = open(tmp_name, "wb")
				imgfile = gzip.open(fname + ".gz", "rb")
				data = imgfile.read()
				tmpfile.write(data)
				tmpfile.close()
				imgfile.close()
				img = Blender.Image.Load(tmp_name)
				os.remove(tmp_name)
			except IOError:			
				Blender.Draw.PupMenu("Error: Could not load image file: " + fname)
				remove(tmp_name)
				Exit()
				return
		if (has_extra_uv):
			fname = Blender.sys.dirname(filename) + Blender.sys.dirsep + material.extra_material_name
			try:
				img2 = Blender.Image.Load(fname)
				compress = False
			except IOError:
				compress = True
			if (compress):
				tmp_name = gettempdir() + Blender.sys.dirsep + material.extra_material_name
				try:
					tmpfile = open(tmp_name, "wb")
					imgfile = gzip.open(fname + ".gz", "rb")
					data = imgfile.read()
					tmpfile.write(data)
					tmpfile.close()
					imgfile.close()
					img2 = Blender.Image.Load(tmp_name)
					os.remove(tmp_name)
				except IOError:			
					Blender.Draw.PupMenu("Error: Could not load image file: " + fname)
					remove(tmp_name)
					Exit()
					return
		else:
			img2 = 0

		i = material.start
		while (i < (material.start + material.count)):
			idx1 = el3d_file_obj.index_list[i + 0]
			idx2 = el3d_file_obj.index_list[i + 1]
			idx3 = el3d_file_obj.index_list[i + 2]
			hash_value = vertex_hash(el3d_file_obj.vertex_list[idx1].position)
			idx4 = vert_dict[hash_value]
			hash_value = vertex_hash(el3d_file_obj.vertex_list[idx2].position)
			idx5 = vert_dict[hash_value]
			hash_value = vertex_hash(el3d_file_obj.vertex_list[idx3].position)
			idx6 = vert_dict[hash_value]
			faces.append([[idx4 + 1, idx5 + 1, idx6 + 1], [idx1, idx2, idx3], [img, img2]])
			i += 3
	face_mapping = mesh.faces.extend([f[0] for f in faces], indexList = True)
	mesh_faces = mesh.faces
	mesh.addUVLayer(default_uv_layer)
	if (has_extra_uv):
		mesh.addUVLayer(extra_uv_layer)
	mesh.activeUVLayer = default_uv_layer
	mesh.faceUV = True
	mesh.vertexColors = has_color

	step = len(faces) / 100.0
	pos = 0.0
	next = 0.0
	for i, face in enumerate(faces):
		if (pos >= next):
			Blender.Window.DrawProgressBar(pos / len(faces), 'Importing e3d mesh')
			next += step
		pos += 1.0
		face_index_map = face_mapping[i]
		if (face_index_map != None): # None means the face wasnt added
			blender_face = mesh_faces[face_index_map]
			idx = face[1]
			index = [0] * len(idx)
			for i in range(0, len(idx)):
				blender_face.uv[i][0] = el3d_file_obj.vertex_list[idx[i]].uv[0]
				blender_face.uv[i][1] = el3d_file_obj.vertex_list[idx[i]].uv[1]
				index[i] = normals_hash(el3d_file_obj.vertex_list[idx[i]].normal)
				if (has_color):
					blender_face.col[i].r = el3d_file_obj.vertex_list[idx[i]].color[0]
					blender_face.col[i].g = el3d_file_obj.vertex_list[idx[i]].color[1]
					blender_face.col[i].b = el3d_file_obj.vertex_list[idx[i]].color[2]
					blender_face.col[i].a = el3d_file_obj.vertex_list[idx[i]].color[3]
				if (has_extra_uv):
					mesh.activeUVLayer = extra_uv_layer
					blender_face.uv[i][0] = el3d_file_obj.vertex_list[idx[i]].extra_uvs[0]
					blender_face.uv[i][1] = el3d_file_obj.vertex_list[idx[i]].extra_uvs[1]
					mesh.activeUVLayer = default_uv_layer
			if ((index[0] == index[1]) and (index[1] == index[2])):
				blender_face.smooth = False
			else:
				blender_face.smooth = True
			blender_face.image = face[2][0]
			if (has_extra_uv):
				mesh.activeUVLayer = extra_uv_layer
				blender_face.image = face[2][1]
				mesh.activeUVLayer = default_uv_layer
	mesh.verts.delete(0)
	scene = Blender.Scene.GetCurrent()
	scene.objects.new(mesh)
	Blender.Redraw()
	print "Done e3d import from file: ", filename

def select_load_el3d(filename):
	dir_name = os.path.dirname(filename)
	load_dir = Blender.Draw.PupMenu("Load all files in dir '" + dir_name + "' or file '" + filename + "'?%t|Load dir|Load file")
	if (load_dir == 1):
		file_list = os.listdir(dir_name)
		for file_name in file_list:
			if (os.path.isfile(os.path.join(dir_name, file_name))):
				fname = file_name
				name, ext = fname.rsplit('.', 1)
				if (ext == "gz"):
					fname = name
					name, ext = fname.rsplit('.', 1)
				if (ext == "e3d"):
					try:
						load_el3d(os.path.join(dir_name, file_name))
					except el3d_error, (msg):
						PupMenu("Error: '" + msg + "' while loadig file" +
							os.path.join(dir_name, file_name))
	else:
		try:
			load_el3d(filename)
		except el3d_error, (msg):
			PupMenu("Error: '" + msg + "' while loadig file" + file_name)
Blender.Window.FileSelector(select_load_el3d, "Import e3d")
